---[[	    	CURRENT VERSION : BETA 0.3.6   		   ]]---
---[[Copyright (c) Phoenix Digital Media/Gaming/Studios]]---

-- Loads the variables and graphics and sets the gameplay screen height and width
function love.load ()
	monitorWidth = 1024
	monitorHeight = 768
	love.graphics.setMode(monitorWidth, monitorHeight, false, true, 3)
	
	paddleWidth = 30
	paddleHeight = 140
	
	paddle = {}
	paddle.x = monitorWidth/7 - paddleWidth/2
	paddle.y = monitorHeight/2 - paddleHeight/2
	
	paddle2 = {}
	paddle2.x = (monitorWidth * 6)/7 - paddleWidth/2
	paddle2.y = paddle.y
	
	paddleSpeed = 250
	paddleCurve = 1.25
	
	ball = {}
	ball.x = monitorWidth/2 - 15
	ball.y = monitorHeight/2 - 15
	ball.vx = 200
	ball.vy = 0
	
	lostFocus = {}
	lostFocus.x = monitorWidth/4
	lostFocus.y = monitorHeight/4
	focus = true
	drawingFocus = false
	
	intro = {}
	intro.x = lostFocus.x
	intro.y = lostFocus.y
	introB = true -- Note: B stands for Boolean, a type of variable that stores "true" or "false"
	drawingIntro = false
	
	paused = {}
	paused.x = lostFocus.x
	paused.y = lostFocus.y
	pausedB = false
	
	ins = {}
	ins.x = lostFocus.x
	ins.y = lostFocus.y
	insB = false
	
	game = false
	
	player1score = 0
	player2score = 0
	
	paddle1bounceup = false
	paddle2bounceup = false
	paddle1bouncedown = false
	paddle2bouncedown = false
	
	playspeed = 1
	
	debugging = false
	debugInformation = "Preparing to debug"
	deltaTime = 0
	
	version = "0.3.6"
	
	scoreFont = love.graphics.newFont("verdana.ttf", 40)
	versionFont = love.graphics.newFont("verdana.ttf", versionFontSize)
	versionFontSize = 17
	versionFontOffset = ((16 * versionFontSize) + (versionFontSize * 5))/4
	debugFont = love.graphics.newFont("cour.ttf", 15)
	
	ballGraphic = love.graphics.newImage("ball.png")
	lostFocusGraphic = love.graphics.newImage("lostfocus.png")
	introGraphic = love.graphics.newImage("intro.png")
	pausedGraphic = love.graphics.newImage("paused.png")
	titleGraphic = love.graphics.newImage("title.png")
	instructionsGraphic = love.graphics.newImage("instructions.png")
end

-- Checks if a specific mouse button is released
function love.mousereleased(x, y, button)
	-- If the button released is the left mouse button, the game will start, if the right one, it will reset
	if button == "l" and insB == false then
		game = true
		introB = false
	end
	if button == "r" then
		game = false
		resetGame()
	end
end

-- Checks if a certain key is released
function love.keyreleased(key)
	if key == "p" then
		pauseGame()
	end
	-- Pressing "q" will slow down gameplay, "e" will speed it up back to it's normal speed
	if playspeed >= 0.2 then
		if key == "q" then
			playspeed = playspeed - 0.1
		end
	end
	if playspeed < 1 then
		if key == "e" then
			playspeed = playspeed + 0.1
		end
	end
	if playspeed > 1 then
		playspeed = 1
	end
	if key == "b" and debugging == false then
		debugging = true
	elseif key =="b" and debugging == true then
		debugging = false
	end
	if key == "i" and insB == false then
		showInsScreen()
	elseif key == "i" and insB == true then
		insB = false
		resumeGame()
	end
end

-- Checks if the screen is on focus
function love.focus(f)
	-- If it's not on focus, it will pause and display the dialog box, else, it will keep on going
	if not f then
		focus = false
		pauseGame()
	else
		focus = true
	end
end

-- Updates the game and variables
function love.update (dt)
	if game == true then -- Checks if the game is still playing. If not, it will pause or reset 
		-- Checks if a specific key is pressed
		if love.keyboard.isDown("up") then
			paddle.y = paddle.y - (paddleSpeed * dt) * playspeed
		elseif love.keyboard.isDown("down") then
			paddle.y = paddle.y + (paddleSpeed * dt) * playspeed
		end
		if love.keyboard.isDown("w") then
			paddle2.y = paddle2.y - (paddleSpeed * dt) * playspeed
		elseif love.keyboard.isDown("s") then
			paddle2.y = paddle2.y + (paddleSpeed * dt) * playspeed
		end
		
		-- Moves the ball to it's velocity
		ball.x = ball.x + (ball.vx * dt) * playspeed
		ball.y = ball.y + (ball.vy * dt) * playspeed
		
		-- Checks collision with the ball and the paddles
		if paddle1bounceup == false or paddle1bouncedown == false then
			if ball.x < paddle.x + paddleWidth and ball.x > paddle.x and ball.y > paddle.y and ball.y < paddle.y + paddleHeight then
				ball.x = paddle.x + paddleWidth
				ball.vx = ball.vx * -1
				ball.vy = ((ball.y + 15) - (paddle.y + paddleHeight/2)) * paddleCurve
			end
		end
		
		if paddle1bounceup == true then
			if ball.y > paddle.y + 30 and ball.x > paddle.x - 15 and ball.x < paddle.x + 30 and ball.y < paddle.y + 2 then
				ball.y = paddle.y
				ball.vy = ball.vy * -1
			end
		end
		
		if paddle1bouncedown == true then
			if ball.y < paddle.y + paddleHeight and ball.x > paddle.x - 15 and ball.x < paddle.x + 30 and ball.y > paddle.y + (paddleHeight - 2) then
				ball.y = paddle.y + paddleHeight
				ball.vy = ball.vy * -1
			end
		end
		
		if ball.x > paddle.x - 30  and ball.x < paddle.x + 60 and ball.y < paddle.y - 15 then
			paddle1bounceup = true
		else
			paddle1bounceup = false
		end
		
		if ball.x > paddle.x - 30  and ball.x < paddle.x + 60 and ball.y > paddle.y then
			paddle1bouncedown = true
		else
			paddle1bouncedown = false
		end
		
		if paddle2bounceup == false or paddle2bounce then
			if ball.x > paddle2.x - paddleWidth and ball.x < paddle2.x and ball.y > paddle2.y and ball.y < paddle2.y + paddleHeight then
				ball.x = paddle2.x - paddleWidth
				ball.vx = ball.vx * -1
				ball.vy = ((ball.y + 15) - (paddle2.y + paddleHeight/2)) * paddleCurve
			end
		end
		
		if paddle2bounceup == true then
			if ball.y > paddle2.y and ball.x - 30 > paddle2.x and ball.x < paddle2.x + 30 and ball.y < paddle2.y + paddleHeight then
				ball.y = paddle2.y
				ball.vy = ball.vy * -1
			end
		end
		
		if paddle2bouncedown == true then
			if ball.y < paddle2.y + paddleHeight and ball.x - 30 > paddle2.x and ball.x < paddle2.x + 30 and ball.y > paddle2.y then
				ball.y = paddle2.y + paddleHeight
				ball.vy = ball.vy * -1
			end
		end
		
		if ball.x > paddle2.x - 30 and ball.x < paddle2.x + 60 and ball.y < paddle2.y - 15 then
			paddle2bounceup = true
		else
			paddle2bounceup = false
		end
		
		if ball.x > paddle2.x - 30 and ball.x < paddle2.x + 60 and ball.y > paddle2.y then
			paddle2bouncedown = true
		else
			paddle2bouncedown = false
		end
		
		-- Adds boundaries, so that when it hits the top, or bottom sides of the screen, it will bounce it back
		if ball.y == 0 or ball.y < 0 then
			ball.y = 2
			ball.vy = ball.vy * -1
		end
		
		if ball.y == monitorHeight - 30 or ball.y > monitorHeight - 30 then
			ball.y = monitorHeight - 32
			ball.vy = ball.vy * -1
		end
		
		-- Checks if the ball is at the left, or right side of the screen
		if ball.x < -30 then
			ball.x = monitorWidth/2 - 15
			ball.y = monitorHeight/2 - 15
			player2score = player2score + 1
		elseif ball.x > monitorWidth + 30 then
			ball.x = monitorWidth/2 - 15
			ball.y = monitorHeight/2 - 15
			player1score = player1score + 1
		end
		
		-- Adds boundaries to the paddle, so that it wont go outside the playing area
		if paddle.y < 0 then
			paddle.y = 0
		elseif paddle.y > monitorHeight - paddleHeight then
			paddle.y = monitorHeight - paddleHeight
		end
		
		if paddle2.y < 0 then
			paddle2.y = 0
		elseif paddle2.y > monitorHeight - paddleHeight then
			paddle2.y = monitorHeight - paddleHeight
		end
	end
	
	-- Updates debug information
	debugInformation = "Debug screen" .. "\nball.vx, ball.vy: " .. ball.vx .. "," .. ball.vy .. " ball.x, ball.y: " .. ball.x .. "," .. ball.y .. " \npaddle.x, paddle.y: " .. paddle.x .. "," .. paddle.y .. " \npaddle2.x, paddle2.y: " .. paddle2.x .. "," .. paddle2.y .. " \ndeltaTime: " .. dt .. " playspeed: " .. playspeed
end

-- Draws the graphics and updates it every frame
function love.draw()
	love.graphics.setColor(255,255,255,255)
	love.graphics.rectangle("fill", paddle.x, paddle.y, paddleWidth, paddleHeight)love.graphics.rectangle("fill", paddle2.x, paddle2.y, paddleWidth, paddleHeight)
	love.graphics.draw(ballGraphic, ball.x, ball.y)
	
	love.graphics.setColor(255,255,255,255)
	if focus == false and introB == false and pausedB == true then
		love.graphics.draw(lostFocusGraphic, lostFocus.x, lostFocus.y)
		drawingFocus = true
	else
		drawingFocus = false
	end
	if introB == true and focus == true then
		love.graphics.draw(introGraphic, intro.x, intro.y)
		drawingIntro = true
	else
		drawingIntro = false
	end
	if game == false and focus == true and pausedB == true and drawingFocus == false and drawingIntro == false then
		love.graphics.draw(pausedGraphic, paused.x, paused.y)
	end
	if game == false and focus == true and pausedB == false and drawingFocus == false and drawingIntro == false and insB == true then
		love.graphics.draw(instructionsGraphic, ins.x, ins.y)
	end
	love.graphics.draw(titleGraphic, monitorWidth/2 - 140, 5)
	love.graphics.setFont(scoreFont)
	love.graphics.print(player1score, monitorWidth/9 - 40, monitorHeight/8)
	love.graphics.print(player2score, (monitorWidth * 8)/9, monitorHeight/8) 
	love.graphics.setFont(versionFont)
	love.graphics.print("Current Version:" .. version, monitorWidth/2 - versionFontOffset, monitorHeight/9) 
	if debugging == true then
		love.graphics.setFont(debugFont)
		love.graphics.print(debugInformation, 20, (monitorHeight * 8)/9)
	end
end

-- Resets game, or pauses it
function resetGame ()
	ball.x = monitorWidth/2 - 15
	ball.y = monitorHeight/2 - 15
	game = false
	paddle.x = monitorWidth/7 - paddleWidth/2
	paddle.y = monitorHeight/2 - paddleHeight/2
	paddle2.x = (monitorWidth * 6)/7 - paddleWidth/2
	paddle2.y = paddle.y
end

function pauseGame()
	game = false
	if insB == true then 
		pausedB = false
	else
		pausedB = true
	end
end

function showInsScreen()
	game = false
	insB = true
end

function resumeGame()
	game = true
	pausedB = false
end